<?php declare(strict_types=1);

use Lewisdale\Webmentions\Endpoint;
use Lewisdale\Webmentions\Exceptions\InvalidTargetException;
use Lewisdale\Webmentions\Exceptions\InvalidUrlException;
use Lewisdale\Webmentions\Exceptions\SourceNotFoundException;
use Lewisdale\Webmentions\Exceptions\TargetNotMentionedException;
use PHPUnit\Framework\TestCase;
use PHPUnit\Framework\Attributes\TestWith;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

class EndpointTest extends TestCase {
    #[TestWith(["https://my.url.com", true])]
    #[TestWith(["my.url.com", false])]
    public function testValidatesUrls(string $url, bool $expected)
    {
        $endpoint = new Endpoint($this->createMock(HttpClientInterface::class));
        $this->assertEquals($expected, $endpoint->validateUrl($url), "Expected $url");
    }

    #[TestWith(["https://my-valid-source.url", "htt://my-invalid-target.com"])]
    #[TestWith(["my-invalid-source", "https://my-valid-target.com"])]
    #[TestWith(["http:///an-invalid-source", "an-invalid-target"])]
    public function testThrowsInvalidUrlExceptionIfTheUrlIsInvalid(string $source, string $target)
    {
        $this->expectException(InvalidUrlException::class);
        $endpoint = new Endpoint($this->createMock(HttpClientInterface::class));
        $endpoint->receiveWebmention($source, $target);
    }

    public function testThrowsInvalidTargetExceptionIfTheTargetUrlIsNotOnCurrentSite()
    {
        $this->expectException(InvalidTargetException::class);
        $endpoint = new Endpoint($this->createMock(HttpClientInterface::class));
        $endpoint->receiveWebmention("https://a-valid-source.url", "https://not-my-site.com");
    }

    #[TestWith([404])]
    #[TestWith([401])]
    #[TestWith([500])]
    #[TestWith([501])]
    #[TestWith([502])]
    #[TestWith([503])]
    #[TestWith([500])]
    public function testItShouldThrowASourceNotFoundExceptionIfTheSourceUrlIsUnreachable(int $statusCode)
    {
        $source = "https://my-valid-source-url.com";
        $target = "https://lewisdale.dev/post/a-post-page";

        $mockClient = $this->createMock(HttpClientInterface::class);
        $mockResponse = $this->createMock(ResponseInterface::class);

        $mockClient->expects($this->once())
            ->method('request')
            ->with($this->identicalTo('GET'), $this->identicalTo($source))
            ->will($this->returnValue($mockResponse));

        $mockResponse->method('getStatusCode')
            ->will($this->returnValue($statusCode));

        $this->expectException(SourceNotFoundException::class);

        $endpoint = new Endpoint($mockClient);
        $endpoint->receiveWebmention($source, $target);
    }

    public function testItShouldThrowATargetNotMentionedErrorIfTheSourceDoesNotMentionTheTarget()
    {
        $source = "https://my-valid-source-url.com";
        $target = "https://lewisdale.dev/post/a-post-page";

        $content = <<<XML
        <html>
            <head>
            </head>
            <body>
                <h1>Some content</h1>
                <p>Here's some body content. It <a href="/another/page">contains a url</a>.</p>
                <p>But it does not mention the target.</p>
            </body>
        </html>
        XML;

        $mockClient = $this->createMock(HttpClientInterface::class);
        $mockResponse = $this->createMock(ResponseInterface::class);

        $mockClient->expects($this->once())
            ->method('request')
            ->with($this->identicalTo('GET'), $this->identicalTo($source))
            ->will($this->returnValue($mockResponse));

        $mockResponse->method('getStatusCode')
            ->will($this->returnValue(200));

        $mockResponse->method('getContent')
            ->willReturn($content);

        $this->expectException(TargetNotMentionedException::class);

        $endpoint = new Endpoint($mockClient);
        $endpoint->receiveWebmention($source, $target);
    }
}
?>